/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
 *  - http://212.187.12.197/RNG/terraform/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * The 4-layerd Perlin noise generation code is taken/adapted from
 * John Ratcliff (jratcliff@verant.com) who placed his code under
 * the public domain.
 */


#include <stdio.h>
#include <math.h>
#include "HeightFieldGenPerlin.h"
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"
#include "MathGauss.h"
#include "string.h"


#define EMPTY	0.0

/*
 *  constructor: initialize
 */
HeightFieldGenPerlin::HeightFieldGenPerlin (HeightField *HF) 
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ HeightFieldGenPerlin\n");

	SanityCheck::bailout ((!HF), "HF==NULL", "HeightFieldGenPerlin::HeightFieldGenPerlin");

	p_HF = HF;
	p_mGauss = NULL;
}


/*
 *  destructor: noting to clean up
 */
HeightFieldGenPerlin::~HeightFieldGenPerlin ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- HeightFieldGenPerlin\n");
}


/*
 *  generate: call Builder to generate the Perlin noise. 
 */
int HeightFieldGenPerlin::generate (int w, int h, int seed, 
				    bool p1, bool p2, bool p3, bool p4,
				    float f1, float f2, float f3, float f4,
				    float a1, float a2, float a3, float a4,
				    int o1, int o2, int o3, int o4,
				    int e1, int e2, int e3, int e4)
{
	int	x,y;

	//p_mGauss = new MathGauss (seed);
	//n = setPreferredSize (n);
	p_HF->newHF (w, h);

	Builder *b = new Builder (w, h, seed, p1, p2, p2, p3,
				 f1, f2, f3, f4, a1, a2, a3, a4,
				 o1, o2, o3, o4, e1, e2, e3, e4);

	// We have to call the get() in this order to avoid re-seeding 
	// scanlines at every get. See Octave::Get()
	for (y=0; y<h; y++)
		{
		//printf ("%d,%d\n", x, y);
		for (x=0; x<w; x++)
			{
			//(printf ("%d,%d\n", x, y);
			p_HF->setEl (x, y, b->Get(x,y));
			}
		}

	p_HF->setSaved (FALSE);
	delete b;

	return (0);
}


/* ********************************************************************** */


/*
 *  Builder constructor: build the Perlin noise
 */
Builder::Builder(int wid, int hit, int seed, bool p1, bool p2, bool p3, bool p4,
		float f1, float f2, float f3, float f4,
		float a1, float a2, float a3, float a4,
		int o1, int o2, int o3, int o4,
		int e1, int e2, int e3, int e4)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ HFGenPerlin:Builder\n");

	mEnvelopes = new Envelopes;

	mWidth  = wid;
	mHeight = hit;
	mSeed = seed;
	mEnvelope1 = (EnvelopeType) e1;
	mEnvelope2 = (EnvelopeType) e2;
	mEnvelope3 = (EnvelopeType) e3;
	mEnvelope4 = (EnvelopeType) e4;

	mPerlin1 = 0;
	mPerlin2 = 0;
	mPerlin3 = 0;
	mPerlin4 = 0;

	if (p1) mPerlin1 = new Octaves(o1,mWidth,mHeight,0,0,f1,0,a1,2.0f,0.5f,seed+0);
	if (p2) mPerlin2 = new Octaves(o2,mWidth,mHeight,0,0,f2,0,a2,2.0f,0.5f,seed+1);
	if (p3) mPerlin3 = new Octaves(o3,mWidth,mHeight,0,0,f3,0,a3,2.0f,0.5f,seed+2);
	if (p4) mPerlin4 = new Octaves(o4,mWidth,mHeight,0,0,f4,0,a4,2.0f,0.5f,seed+3);

	srand(seed);
	int rx,ry;

	if (mPerlin1)
		{
		rx = rand();
		ry = rand();
		mPerlin1->SetBaseUV(rx,ry);
		mPerlin1->Regenerate();
		}

	if (mPerlin2)
		{
		rx = rand();
		ry = rand();
		mPerlin2->SetBaseUV(rx,ry);
		mPerlin2->Regenerate();
		}

	if (mPerlin3)
		{
		rx = rand();
		ry = rand();
		mPerlin3->SetBaseUV(rx,ry);
		mPerlin3->Regenerate();
		}

	if (mPerlin4)
		{
		rx = rand();
		ry = rand();
		mPerlin4->SetBaseUV(rx,ry);
		mPerlin4->Regenerate();
		}
}


/*
 *  Builder destructor: clean up
 */
Builder::~Builder ()
{
	delete mPerlin1;
	delete mPerlin2;
	delete mPerlin3;
	delete mPerlin4;
	delete mEnvelopes;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- HFGenPerlin:Builder\n");
}


/*
 *  Builder::Get: gets/builds the data for a point in the perlin noise
 */
float Builder::Get (int x, int y)
{

	#define ZRECIP1 (1.0f/64.0f)

	float h1=0;
	float h2=0;
	float h3=0;
	float h4=0;

	if (mPerlin1)
		{
		h1 = mPerlin1->Get(x,y)*ZRECIP1;
		h1 = mEnvelopes->Get(h1,mEnvelope1);
		}

	if (mPerlin2)
		{
		h2 = mPerlin2->Get(x,y)*ZRECIP1;
		h2 = mEnvelopes->Get(h2,mEnvelope2);
		}

	if (mPerlin3)
		{
		h3 = mPerlin3->Get(x,y)*ZRECIP1;
		h3 = mEnvelopes->Get(h3,mEnvelope3);
		}

	if (mPerlin4)
		{
		h4 = mPerlin1->Get(x,y)*ZRECIP1;
		h4 = mEnvelopes->Get(h4,mEnvelope4);
		}

	float h = (h1+h2+h3+h4);
	#define ZRECIP2 (1.0f/8.0f)
	return h*ZRECIP2;
};


/*
 *  Builder::Get: gets/builds the data for a point in the perlin noise
 */
float Builder::Get (int x, int y, float h)
{
	float h1=0;
	float h2=0;
	float h3=0;
	float h4=0;

	if (mPerlin1)
		h1 = mEnvelopes->Get(h, mEnvelope1);

	if (mPerlin2)
		h2 = mEnvelopes->Get(h, mEnvelope2);

	if (mPerlin3)
		h3 = mEnvelopes->Get(h, mEnvelope3);

	if (mPerlin4)
		h4 = mEnvelopes->Get(h, mEnvelope4);

	return h1+h2+h3+h4;
};


/* ********************************************************************** */


/*
 *  Envelopes constructor: 
 */
Envelopes::Envelopes ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ HFGenPerlin:Envelopes\n");

	for (int i=ET_NULL; i<ET_LAST; i++)
		mEnvelopes[i] = Get((EnvelopeType)i);
}


/*
 *  Envelopes destructor: 
 */
Envelopes::~Envelopes ()
{
	for (int i=ET_NULL; i<ET_LAST; i++) 
		delete mEnvelopes[i];

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- HFGenPerlin:Envelopes\n");
}


/*
 *  Envelopes::Get: 
 */
float Envelopes::Get (float v, EnvelopeType t) const
{
	if (mEnvelopes[t]) 
		return mEnvelopes[t]->Get(v);
  	return v;
}


/*
 *  Envelopes::Get: 
 */
Equation *Envelopes::Get (EnvelopeType choice)
{
	Equation *e=0;

	switch (choice)
		{
		case ET_SQUARED1:
		case ET_SQUARED2:
		case ET_SQUARED3:
			{
			float ratio = float(choice-ET_NULL)/3;
			float scaler1 = ratio*0.02f;
			e = new Equation(MF_SQUARED,scaler1,1);
			}
			break;
		case ET_ATAN1:
		case ET_ATAN2:
		case ET_ATAN3:
			{
			float ratio = float(choice-ET_SQUARED3)/3;
			float scaler1 = 90*ratio;
			float amp1 = 0.03f;
			e = new Equation(MF_ATAN,amp1,scaler1);
			}
			break;
		case ET_ACOS1:
		case ET_ACOS2:
		case ET_ACOS3:
		case ET_ACOS4:
			{
			#define T_SCALE3 180
			#define T_AMP3 0.011f
			float ratio = float(choice-ET_ATAN3)/4;
			float scaler1 = T_SCALE3*ratio;
			float amp1 = T_AMP3;
			e = new Equation(MF_ACOS,amp1,scaler1);
			}
			break;
		case ET_COS1:
			{
			#define T_SCALE7 128
			#define T_AMP7 0.015f
			float scaler1 = T_SCALE7;
			float amp1 = T_AMP7;
			e = new Equation(MF_COS,amp1,scaler1);
			}
			break;
		case ET_COS2:
			{
			#define T_SCALE8 128
			#define T_AMP8 0.02f
			float scaler1 = T_SCALE8;
			float amp1 = T_AMP8;
			e = new Equation(MF_COS,amp1,scaler1);
			}
			break;
		case ET_EXP1:
			{
			#define T_SCALE9 128
			#define T_AMP9 0.025f
			float scaler1 = T_SCALE9;
			float amp1 = T_AMP9;
			e = new Equation(MF_COS,amp1,scaler1);
			}
			break;
		case ET_EXP2:
			{
			#define T_SCALE11 80
			#define T_AMP11 0.010f
			float scaler1 = T_SCALE11;
			float amp1 = T_AMP11;
			e = new Equation(MF_EXP,amp1,scaler1);
			}
			break;
		case ET_EXP3:
			{
			#define T_SCALE12 120
			#define T_AMP12 0.013f
			float scaler1 = T_SCALE12;
			float amp1 = T_AMP12;
			e = new Equation(MF_EXP,amp1,scaler1);
			}
			break;
		case ET_EXP4:
			{
			#define T_SCALE13 160
			#define T_AMP13 0.018f
			float scaler1 = T_SCALE13;
			float amp1 = T_AMP13;
			e = new Equation(MF_EXP,amp1,scaler1);
			}
			break;
		case ET_EXP5:
			{
			#define T_SCALE14 600
			#define T_AMP14 0.055f
			float scaler1 = T_SCALE14;
			float amp1 = T_AMP14;
			e = new Equation(MF_EXP,amp1,scaler1);
			}
			break;
		case ET_LOG1:
			{
			#define T_SCALE15 30
			#define T_AMP15 0.12f
			float scaler1 = T_SCALE15;
			float amp1 = T_AMP15;
			e = new Equation(MF_LOG,amp1,scaler1,-4,256);
			}
			break;
		case ET_LOG2:
			{
			#define T_SCALE16 70
			#define T_AMP16 0.03f
			float scaler1 = T_SCALE16;
			float amp1 = T_AMP16;
			e = new Equation(MF_LOG,amp1,scaler1,-15,256);
			}
			break;
		case ET_LOG3:
			{
			#define T_SCALE17 60
			#define T_AMP17 0.10f
			float scaler1 = T_SCALE17;
			float amp1 = T_AMP17;
			e = new Equation(MF_LOG,amp1,scaler1,-1,70);
			}
			break;
		case ET_TANH:
			{
			#define T_SCALE18 64
			#define T_AMP18 0.027f
			float scaler1 = T_SCALE18;
			float amp1 = T_AMP18;
			e = new Equation(MF_TANH,amp1,scaler1,-256,256);
			}
			break;
		case ET_SQRT:
			{
			#define T_SCALE19 256
			#define T_AMP19 0.009f
			float scaler1 = T_SCALE19;
			float amp1 = T_AMP19;
			e = new Equation(MF_SQRT,amp1,scaler1,30,256);
			}
			break;
		case ET_NULL:
		case ET_LAST:
				break;
		}
		
	return e;
}


/* ********************************************************************** */


/*
 *  Equation constuctor
 */
Equation::Equation (EquationType type, float constant, float scaler,
		    float minv, float maxv)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ HFGenPerlin:Equation\n");

	mFunction	= type;
	mConstant	= constant;
	mScaler		= scaler;
	mMin		= minv;
	mMax		= maxv;
};


/*
 *  Equation constuctor
 */
Equation::~Equation ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- HFGenPerlin:Equation\n");
}


/*
 *  Equation::Get: gets the perlin value transformed by the specified equation
 */
float Equation::Get (float v) const
{
	switch (mFunction)
		{
		case MF_SQUARED:
			v = v*v*mConstant*mScaler;
			break;
		case MF_SIN:
			v = (float)sin(v*mConstant)*mScaler;
			break;
		case MF_COS:
			v = (float)cos(v*mConstant)*mScaler;
			break;
		case MF_TAN:
			v = (float)tan(v*mConstant)*mScaler;
			break;
		case MF_SINH:
			v = (float)sinh(v*mConstant)*mScaler;
			break;
		case MF_COSH:
			v = (float)cosh(v*mConstant)*mScaler;
			break;
		case MF_TANH:
			v =(float)tanh(v*mConstant)*mScaler;
			break;
		case MF_ASIN:
			{
			float i = v*mConstant;
			if (i < -1) i = -1;
			if (i >  1) i = 1;
			v = (float)asin(i)*mScaler;
			}
			break;
		case MF_ACOS:
			{
			float i = v*mConstant;
			if (i < -1) i = -1;
			if (i >  1) i = 1;
			v = (float)acos(i)*mScaler;
			}
			break;
		case MF_ATAN:
			v = (float)atan(v*mConstant)*mScaler;
			break;
		case MF_EXP:
			v = (float)exp(v*mConstant)*mScaler;
			break;
		case MF_LOG:
			v = (float)log(fabs(v*mConstant))*mScaler;
			break;
		case MF_SQRT:
			v = (float)sqrt(fabs(v*mConstant))*mScaler;
			break;
		case MF_LAST:
			break;
		}
		
	if ( v < mMin ) v = mMin;
	if ( v > mMax ) v = mMax;

	return v;
};


/* ********************************************************************** */

#define PRECISION 16

/*
 *  FastPerlin constructor
 */
FastPerlin::FastPerlin(int seed)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ HFGenPerlin:FastPerlin\n");

	MathGauss	r (seed);

	mRandom1 = new short[TABLESIZE];
	mRandom2 = new short[TABLESIZE];
	for (int i=0; i<TABLESIZE; i++)
		{
		mRandom1[i] = r.gauss()*2;
		mRandom2[i] = r.gauss();
		}
}


/*
 *  FastPerlin destructor
 */
FastPerlin::~FastPerlin(void)
{
	delete mRandom1;
	delete mRandom2;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- HFGenPerlin:FastPerlin\n");
}


/* 
 * Procedure C_Noise()
 * Inputs: u_init, v_init: starting u and v parameters into the image.
 *	du, dv: the incremental change to u_init and v_init.
 *	ddu, ddv: the incremental change to du and dv.
 *	Num_Pix: number of pixels in the scan line to draw.
 *	screen_buffer: Pointer to the screen buffer.
 *	Output: 16 bit pixels are drawn to the screen.
 */
void FastPerlin::Noise(int u_init, int v_init, int du, int Num_Pix,
             		short* screen_buffer)

{
	unsigned char 	bx0, bx1, by0, by1;
	unsigned char 	rx0, ry0;

	//FOR ALGORITHM BASED COMMENTS - PLEASE SEE ORIGINAL PROGRAM IN
	//SECTION 5.
	//Original Perlin's noise program used an array g[512][2].
	//This program replaces the array with real-time calculations.
	//The results are stored into variables of the form g_b##_#.

	short 	g_b00_0; //Used to replace array g[b00][0]
	short 	g_b00_1; //Used to replace array g[b00][1]
	short 	g_b01_0; //replaces array g[b01][0]
	short 	g_b01_1; //replaces array g[b01][1]
	short 	g_b10_0; //replaces array g[b10][0]
	short 	g_b10_1; //replaces array g[b10][1]
	short 	g_b11_0; //replaces array g[b11][0]
	short 	g_b11_1; //replaces array g[b11][1]


	//Used to replace array: b00 = p[p[bx0] + by0];
	short 	b00;
	short 	b01;
	short 	b10;
	short 	b11;

	unsigned short 	i;
	unsigned short 	u_16bit, v_16bit;
	signed   short 	rx1, ry1;
	signed   short 	sx, sy;
	signed   int 	u1, v1, u2, v2;
	signed   int 	a, b;

	//Inner loop for the scan line.  "Num_Pix" pixels will be drawn.
	for (i = Num_Pix; i; i--)
		{
		//Convert the u and v parameters from 10.22 to 8.8 format.
		u_16bit = (short) (u_init >> 14);
		v_16bit = (short) (v_init >> 14);

		//Same as Perlin's original code except floating point
		//is converted over to fixed integer.

		bx0 = u_16bit >> 8;		//Obtain the integer part of the u coordinate
		bx1 = bx0 + 1;
		rx0 = u_16bit & 255;   	//Obtain the fractional part of the u coordinate
		rx1 = rx0 - 256;
		by0 = v_16bit >> 8;
		by1 = by0 + 1;
		ry0 = v_16bit & 255;
		ry1 = ry0 - 256;

		//Same as Perlin's original code except floating point
		//is converted over to fixed integer.  The ">> 1" is used to
		//avoid overflow when the values are multiplied together.
		//This is not a problem in "C" but will be in the MMX implementation.

		sx = (((rx0 * rx0) >> 1) * ((1536 - (rx0 << 2)))) >> 16;
		sy = (((ry0 * ry0) >> 1) * ((1536 - (ry0 << 2)))) >> 16;

		//This is where the code differs from Perlins.
		//Rather than use arrays p[] and g[][], the values are
		//calculated real-time.  Here random1() replaces array p[].
		//Perlin's equivalent: b00 = p[p[bx0] + by0];

		b00 = random1((random1(bx0) + by0));
		b01 = random1((random1(bx0) + by1));
		b10 = random1((random1(bx1) + by0));
		b11 = random1((random1(bx1) + by1));

		//Here, random2() replaces array g[][].
		//Perlin's equivalent: g_b00_0 = g[b00][0];

		g_b00_0 = (random2(b00) & 511) - 256;
		g_b01_0 = (random2(b01) & 511) - 256;
		g_b10_0 = (random2(b10) & 511) - 256;
		g_b11_0 = (random2(b11) & 511) - 256;
		g_b00_1 = (random2((b00 + 1)) & 511) - 256;
		g_b01_1 = (random2((b01 + 1)) & 511) - 256;
		g_b10_1 = (random2((b10 + 1)) & 511) - 256;
		g_b11_1 = (random2((b11 + 1)) & 511) - 256;


		//Same as Perlin's original code
		u1 = rx0 * g_b00_0 + ry0 * g_b00_1;
		v1 = rx1 * g_b10_0 + ry0 * g_b10_1;
		a = u1 + sx * ((v1 - u1) >> 8);
		//Same as Perlin's original code
		u2 = rx0 * g_b01_0 + ry1 * g_b01_1;
		v2 = rx1 * g_b11_0 + ry1 * g_b11_1;
		b = u2 + sx * ((v2 - u2) >> 8);

		//Same as Perlin's original code except the output is
		//converted from fixed point to regular integer.  Also
		//since the output ranges from -256 to +256, a 256 offest
		//is added to make the range from 0 to 511.  This offset
		//is the 65536 value.  Then the 0 to 511 is scaled down.
		//to a range of 0 to 255.
		short color = (short) ((a + 65536 + sy * ((b - a) >> 8)) >> 4);
		color-=(256*PRECISION);
		*(screen_buffer) = color;

		u_init += du;    // New u for calc the color of the next pixel
		screen_buffer++; // Advance 1 word

		}	// End of the scanline - repeat for next pixel
}


/* ********************************************************************** */

/*
 *  Octave constcutor
 */
Octave::Octave (int wid, int hit, int u, int v, int du,
         	int dv, int amplitude, int seed)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ HFGenPerlin:Octave\n");

	mWidth		= wid;
	mHeight		= hit;
	mU		= u;
	mV		= v;
	mDU		= du;
	mDV		= dv;
	mAmplitude 	= amplitude;
	mScanY		= -1;
	mScanLine 	= new short[wid];
	mPerlin 	= new FastPerlin(seed);
};


/*
 *  Octave destrutor
 */
Octave::~Octave()
{
	delete mScanLine;
	delete mPerlin;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- HFGenPerlin:Octave\n");
}


/*
 *  Octave::Generate: generates the noise for 1 scanline 
 */
void Octave::Generate (int y)
{
	mScanY  = y;
	int v   = mV+(y*mDV);
	mPerlin->Noise (mU, v, mDU, mWidth, mScanLine);
}

/*
 *  Octave::Get: gets the noise at a given point. If we call a scan line which 
 * 	is not in the data buffer, the scan line is generated on the fly. This 
 * 	means that it's essential to call this method in the correct order so 
 * 	that you process all X offsets of a gives scanline (Y) before moving 
 *	on to the next scanline (Y).
 */
int Octave::Get (int x, int y)
{
	//assert( y >= 0 && y < mHeight );
	if (y != mScanY)
		Generate (y);
	//assert ( x >= 0 && x < mWidth );
	int v = (int)mScanLine[x];
	int ret = ((v*mAmplitude)>>12);
	return ret;
}

/*
 *  Octave::Set: sets the parameters
 */
void Octave::Set (int w, int h, int u, int v, int du, int dv, int ampl)
{
	if (w != mWidth)
		{
		delete mScanLine;
		mScanLine = new short[mWidth];
		}

	mWidth = w;
	mHeight = h;
	mU = u;
	mV = v;
	mDU = du;
	mDV = dv;
	mAmplitude = ampl;
}


/* ********************************************************************** */

/*
 *  Octaves constuctor: build all the octaves which we need for the perlin noise
 */
Octaves::Octaves (int octaves, int wid, int hit, int u, int v, float freqf,
          int base, int amplitude, float freqdecay, float ampdecay, int seed)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ HFGenPerlin:Octaves\n");

	int i, freq 	= (int) (freqf*(float)(1<<20));
	mFreqDecay	= freqdecay;
	mAmpDecay	= ampdecay;
	mWidth  	= wid;
	mHeight 	= hit;
	mU      	= u;
	mV      	= v;
	mFreq   	= freq;
	mBase   	= base;
	mAmplitude 	= amplitude;
	mOctaveCount 	= octaves;

	for (i=0; i<octaves; i++)
		{
		int 	u = mU*freq;
		int 	v = mV*freq;
		Octave *o = new Octave (wid,hit,u,v,freq,freq,amplitude,seed);
		mOctaves.push_back (o);
		freq      = (int) ( (float) freq * mFreqDecay );
		amplitude = (int) ( (float) amplitude * mAmpDecay );
		seed++;
		}
}

/*
 *  Octaves destuctor
 */
Octaves::~Octaves ()
{
	OctaveVector::iterator i;
	for (i=mOctaves.begin(); i!=mOctaves.end(); ++i)
		{
		Octave *o = (*i);
		delete o;
		}

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- HFGenPerlin:Octaves\n");
}


/*
 *  Octaves::Get: get the sum of the octaves for a particular data point
 */
int Octaves::Get (int x, int y)
{
	int total = 0;
	OctaveVector::iterator i;
	int count = 0;
	for (i=mOctaves.begin(); i!=mOctaves.end() && count < mOctaveCount; 
	     ++i, count++)
		total+=(*i)->Get(x,y);
	total+=mBase;
	return total;
};


/*
 *  Octaves::Regenerate
 */
void Octaves::Regenerate ()
{
	OctaveVector::iterator 	i;
	int 			freq = mFreq;
  	int 			amplitude = mAmplitude;

	for (i=mOctaves.begin(); i!=mOctaves.end(); ++i)
		{
		int 	u = mU*freq;
		int 	v = mV*freq;
		Octave *o = (*i);
		o->Set (mWidth,mHeight,u,v,freq,freq,amplitude);

		freq      = (int) ( (float) freq * mFreqDecay);
		amplitude = (int) ( (float) amplitude * mAmpDecay);
  		}
}

