/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
 *  - http://212.187.12.197/RNG/terraform/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef _HEIGHT_FIELD_GEN_PERLIN_H
#define _HEIGHT_FIELD_GEN_PERLIN_H

/* 
 * The 4-layerd Perlin noise generation code is taken/adapted from
 * John Ratcliff (jratcliff@verant.com) who placed his code under
 * the public domain.
 */


#define TABLESIZE 1024
#define TABLEMASK (TABLESIZE-1)

#include "HeightField.h"
#include "MathGauss.h"

enum EquationType
{
	MF_SQUARED,
	MF_SIN,
	MF_COS,
	MF_TAN,
	MF_SINH,
	MF_COSH,
	MF_TANH,
	MF_ASIN,
	MF_ACOS,
	MF_ATAN,
	MF_EXP,
	MF_LOG,
	MF_SQRT,
	MF_LAST
};

enum EnvelopeType
{
	ET_NULL,      // 0 do nothing, leave it as is.
	ET_SQUARED1,  // 1
	ET_SQUARED2,  // 2
	ET_SQUARED3,  // 3
	ET_ATAN1,     // 4 (arc tangent 1)
	ET_ATAN2,     // 5 (arc tangent 2)
	ET_ATAN3,     // 6 (arc tangent 3)
	ET_ACOS1,     // 7 (arc cosine 1)
	ET_ACOS2,     // 8 (arc cosine 2)
	ET_ACOS3,     // 9 (arc cosine 3)
	ET_ACOS4,     // 10 (arc cosine 4)
	ET_COS1,      // 11
	ET_COS2,      // 12
	ET_EXP1,      // 13
	ET_EXP2,      // 14
	ET_EXP3,      // 15
	ET_EXP4,      // 16
	ET_EXP5,      // 17
	ET_LOG1,      // 18
	ET_LOG2,      // 19
	ET_LOG3,      // 20
	ET_TANH,      // 21 (hyperbolic tangent)
	ET_SQRT,      // 22
	ET_LAST
};


/*
 *  HeightFieldGenPerlin: the class which serves as an interface to terraform
 */
class HeightFieldGenPerlin
	{
	public:
				HeightFieldGenPerlin (HeightField *HF);
				~HeightFieldGenPerlin();
		int		generate (int wid, int hit, int seed, 
					bool p1, bool p2, bool p3, bool p4,
					float f1, float f2, float f3, float f4,
					float a1, float a2, float a3, float a4,
					int o1, int o2, int o3, int o4,
					int e1, int e2, int e3, int e4);

	private:
		HeightField		*p_HF;
		MathGauss		*p_mGauss;

	};


class Octaves;					// forward declaration
class Envelopes;				// forward declaration


/*
 *  Builder: builds the 4-layerd Perlin noise model
 */
class Builder
{
	public:
				Builder (int wid, int hit, int seed, 
					bool p1, bool p2, bool p3, bool p4,
					float f1, float f2, float f3, float f4,
					float a1, float a2, float a3, float a4,
					int o1, int o2, int o3, int o4,
					int e1, int e2, int e3, int e4);
				~Builder(void);
		float 		Get(int x,int y);
		float 		Get(int x,int y,float h);

	private:
		int 			mWidth,
					mHeight,
					mSeed;
		Octaves 		*mPerlin1,
					*mPerlin2,
					*mPerlin3,
					*mPerlin4;
		Envelopes 		*mEnvelopes;
		EnvelopeType 		mEnvelope1,
					mEnvelope2,
					mEnvelope3,
					mEnvelope4;
};



/*
 *  Equation: wraps the equation used to transform the perlin noise signal
 */
class Equation
{
	public:
				Equation (EquationType type, float constant,
					float scaler, float minv=-256, 
					float maxv=256);
				~Equation ();
		float 		Get (float v) const; // compute it using function.

	private:
		EquationType 		mFunction;
		float 			mMin,
					mMax,
					mConstant,
					mScaler;
};


/*
 *  Envelopes: wraps the Equation class to provide easy access
 */
class Envelopes
{
	public:
				Envelopes ();
				~Envelopes ();

		float 		Get (float v, EnvelopeType t) const;

	private:
		Equation 	*Get (EnvelopeType t);
		Equation 	*mEnvelopes[ET_LAST];
};


/*
 *  FastPerlin: generates a single perlin noise signal
 */
class FastPerlin
	{
	public:
  				FastPerlin (int seed);
  				~FastPerlin ();
  		void 		Noise (int u_init, int v_init, int du,
             				int Num_Pix, short* screen_buffer);

	private:
		short 		random1(int i) { return mRandom1[ i&TABLEMASK ]; };
  		short 		random2(int i) { return mRandom2[ i&TABLEMASK ]; };

		short 			*mRandom1;
		short 			*mRandom2;
	};


/*
 *  Octave: holds 1 iteration of the perlin noise
 */
class Octave
	{
	public:
  				Octave (int wid, int hit, int u, int v,
         				int du, int dv, int amplitude, int seed);
  				~Octave ();
		int 		Get (int x,int y);
		void 		Generate (int y); // generate for this scanline.
		void 		Set (int wid, int hit, int u, int v,
				     int du, int dv, int amplitude);

		int 		GetWidth () const 	{ return mWidth; };
		int 		GetHeight () const 	{ return mHeight; };

	private:
		int 			mAmplitude;	// amplitude
		int 			mWidth;
		int 			mHeight;
		int 			mU;
		int 			mV;
		int 			mDU;
		int 			mDV;
		int 			mScanY;		// current scan line generated.
		short 			*mScanLine;	// results of current scanline.
		FastPerlin 		*mPerlin;
	};


#include <vector>
typedef std::vector< Octave * > OctaveVector;


/*
 *  Octave: provides a wrapper for all the octaves the perlin noise consists of
 */
class Octaves
	{
	public:
				Octaves (int octaves, int wid, int hit, int u,
          				int v, float freq, int base, int amplitude,
          				float freqdecay=2.0f, float ampdecay=0.5f,
          				int seed=0);
				~Octaves ();

		int 		Get (int x,int y);
		void 		Regenerate ();

		void 		SetBaseUV (int u,int v)	{ mU = u; mV = v; };
		void 		SetAmplitude (float a)	{ mAmplitude = int(a); };
		void 		SetFrequency (float f)	{ mFreq = int(f*(float)(1<<20)); };
		void 		SetOctaves (int c)	{ mOctaveCount= c; };
		void 		SetFreqDecay (float f)	{ mFreqDecay = f; };
		void 		SetAmpDecay (float a)	{ mAmpDecay = a; };

	private:
		int 			mOctaveCount;
		int 			mBase;
		int 			mWidth;
		int 			mHeight;
		int 			mU;
		int 			mV;
		int 			mFreq;
		int 			mAmplitude;
		float 			mFreqDecay;
		float 			mAmpDecay;
		OctaveVector 		mOctaves;
};


#endif // _HEIGHT_FIELD_GEN_PERLIN_H

